﻿using PSFNetObjects;
using Shared.Models;
using System;
using System.Collections.ObjectModel;
using System.Data;

namespace Shared
{
    /// <summary>
    /// An implementation of IDatabaseAccess that provides access to a PSF database.
    /// </summary>
    public class PsfDatabaseAccess : IDatabaseAccess, IDisposable
    {
        #region Members

        private IPSFSession _session;
        private const int MAGIC = 0x505346;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the current session object.
        /// </summary>
        public IPSFSession Session
        {
            get { return _session; }
            private set
            {
                if (_session != null && _session == value)
                    return;

                _session = value;
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Creates a new instance of PSFDatabaseAccess.
        /// </summary>
        public PsfDatabaseAccess()
        {
            this.Session = new PSFNetSession();
        }

        /// <summary>
        /// Handles this object being destroyed.
        /// </summary>
        ~PsfDatabaseAccess()
        {
            this.Dispose(false);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Refreshes the available list of ledgers.
        /// </summary>
        public static ObservableCollection<User> RefreshUserList(string server)
        {
            if (string.IsNullOrEmpty(server))
                throw new ArgumentNullException("server");

            ObservableCollection<User> userlist = new ObservableCollection<User>();

            foreach (DataRow row in PSFNetSystem.UserList(server).Tables[0].Rows)
                userlist.Add(User.FromDataRow(row));

            return userlist;
        }

        public static ObservableCollection<Database> RefreshDatabaseList(string server, string user)
        {
            if (string.IsNullOrEmpty(server))
                throw new ArgumentNullException("server");

            if (string.IsNullOrEmpty(user))
                throw new ArgumentNullException("user");

            ObservableCollection<Database> databaseList = new ObservableCollection<Database>();

            foreach (DataRow row in PSFNetSystem.DatabaseList(server, "Accounting", user).Tables[0].Rows)
                databaseList.Add(Database.FromDataRow(row));

            return databaseList;
        }

        public static ObservableCollection<Company> RefreshCompanyList(string server, string user, string database)
        {
            if (string.IsNullOrEmpty(server))
                throw new ArgumentNullException("server");

            if (string.IsNullOrEmpty(user))
                throw new ArgumentNullException("user");

            if (string.IsNullOrEmpty(database))
                throw new ArgumentNullException("database");

            ObservableCollection<Company> companyList = new ObservableCollection<Company>();

            foreach (DataRow row in PSFNetSystem.CompanyList(server, database, "Accounting", user).Tables[0].Rows)
                companyList.Add(Company.FromDataRow(row));

            return companyList;
        }

        public bool InitialiseSession(bool isUserWa, string server, string database, string user, string companyCode, string password)
        {
            if (string.IsNullOrEmpty(server) || string.IsNullOrEmpty(database) || string.IsNullOrEmpty(user) || string.IsNullOrEmpty(companyCode))
                return false;

            if (isUserWa == false && string.IsNullOrEmpty(password))
            {
                return false;
            }

            try
            {
                if (isUserWa == false && (PSFNetSystem.ValidateUserPassword(server, user, password) == 0 || PSFNetSystem.ValidateUserPassword(server, user, password) == -1))
                {
                    Utilities.TraceMessage((PSFNetSystem.ValidateUserPassword(server, user, password) == 0 ? "Invalid" : "Expired") + " password");
                    return false;
                }

                Session.Initialise("Accounting", user, companyCode, server, database, MAGIC, false);

                return Session.IsInitialised;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Frees up resources used by this object.
        /// </summary>
        /// <param name="disposing">true if the object is being disposed; otherwise, false.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this._session != null)
                    this._session.Dispose();
            }

            this._session = null;
        }

        /// <summary>
        /// Disposes this object.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
